
package com.btarduinocommunicator;

import android.app.Activity;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothDevice;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.Toast;

import com.btarduinocomunicator.R;


public class MainActivity extends Activity {

    private static final String TAG = "MainActivity";
 


    public static final int MESSAGE_STATE_CHANGE 			= 1;
    public static final int MESSAGE_READ 					= 2;
    public static final int MESSAGE_WRITE 					= 3;
    public static final int MESSAGE_DEVICE_NAME 			= 4;
    public static final int MESSAGE_TOAST 					= 5;


    public static final String DEVICE_NAME 					= "device_name";
    public static final String TOAST 						= "toast";

    // Intent request codes
    private static final int REQUEST_CONNECT_DEVICE_SECURE 	= 1;
    private static final int REQUEST_ENABLE_BT 				= 3;

    //posizioni dei rel all'interno dell'array
    private int K1											= 0;
    private int K2											= 1;
    private int K3											= 2;
    private int K4											= 3;
    
    private int	RELE_OFF									= 0;
    private int RELE_ON										= 1;
    //indica lo stato dei rel
    private int[] aReleStatus								= new int[4];
    
    private Button btnK1									= null;
    private Button btnK2									= null;
    private Button btnK3									= null;
    private Button btnK4									= null;
    
    
    private ImageView imgK1									= null;
    private ImageView imgK2									= null;
    private ImageView imgK3									= null;
    private ImageView imgK4									= null;

    // Name of the connected device
    private String mConnectedDeviceName = null;

    // String buffer for outgoing messages
    private StringBuffer mOutStringBuffer;
    // Local Bluetooth adapter
    private BluetoothAdapter mBluetoothAdapter = null;
  
    private BluetoothService btService = null;


    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
       Log.d(TAG, "onCreate");

      
        setContentView(R.layout.main);
  
        mBluetoothAdapter = BluetoothAdapter.getDefaultAdapter();

        // se il bt non  supportato
        if (mBluetoothAdapter == null) {
            Toast.makeText(this, "Bluetooth is not available", Toast.LENGTH_LONG).show();
            finish();
            return;
        }
        
        initReleStatus();
        
        imgK1	= (ImageView)findViewById(R.id.idReleStatusK1);
        imgK2	= (ImageView)findViewById(R.id.idReleStatusK2);
        imgK3	= (ImageView)findViewById(R.id.idReleStatusK3);
        imgK4	= (ImageView)findViewById(R.id.idReleStatusK4);
    }

    /**
     * inizializza i rele come spenti
     */
    private void initReleStatus(){
    	for (int i=0; i<4; i++){
    		aReleStatus[i]	= RELE_OFF;
    	}
    }
    
    @Override
    public void onStart() {
        super.onStart();
        Log.d(TAG, "onStart");
     
        if (!mBluetoothAdapter.isEnabled()) {
            Intent enableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_ENABLE);
            startActivityForResult(enableIntent, REQUEST_ENABLE_BT);

        } else {
            if (btService == null) enableToSend();
        }
    }

    @Override
    public synchronized void onResume() {
        super.onResume();
        Log.d(TAG, "onResume");

          if (btService != null) {
        
            if (btService.getState() == BluetoothService.STATE_NONE) {
          
              btService.start();
            }
        }
    }

    private String calcNewState(String who, int rele){
    	 
    	 if (aReleStatus[rele] == RELE_ON){
    		 Log.d(TAG, who + " actual status ON --> set to OFF");
    		 //setta il nuovo stato del rel
    		 aReleStatus[rele] = RELE_OFF;
    		 setImages(who, false);
    		 return who + "_" + RELE_OFF + "s";
    	 }else {
    		 Log.d(TAG, who + " actual status OFF --> set to ON");
    		//setta il nuovo stato del rel
    		 aReleStatus[rele] = RELE_ON;
    		 setImages(who, true);
    		 return who + "_" + RELE_ON + "s";
    	 }
    }
    
    private void enableToSend() {
        Log.d(TAG, "enableToSend");
   
        btnK1 = (Button) findViewById(R.id.idBtnK1);
        btnK1.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {

            	Log.d(TAG, "k1 pressed");
            
                sendMessage(calcNewState("k1",K1));
               
            }
        });
        
        btnK2 = (Button) findViewById(R.id.idBtnK2);
        btnK2.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {

                sendMessage(calcNewState("k2",K2));
            }
        });
        
        btnK3 = (Button) findViewById(R.id.idBtnK3);
        btnK3.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                
                sendMessage(calcNewState("k3",K3));
            }
        });
        
        btnK4 = (Button) findViewById(R.id.idBtnK4);
        btnK4.setOnClickListener(new OnClickListener() {
            public void onClick(View v) {
                
                sendMessage(calcNewState("k4",K4));
            }
        });

       
        btService = new BluetoothService(this, mHandler);

    
        mOutStringBuffer = new StringBuffer("");
    }

    @Override
    public synchronized void onPause() {
        super.onPause();
    }

    public void setImages(String who, boolean isOn){
    	if (who.equals("k1")){
    		setImage(imgK1, isOn);
    	}else if (who.equals("k2")){
    		setImage(imgK2, isOn);
    	}else if (who.equals("k3")){
    		setImage(imgK3, isOn);
    	}else if (who.equals("k4")){
    		setImage(imgK4, isOn);
    	}
    }
    
    private void setImage(ImageView img, boolean isOn){
    	if (isOn){
    		img.setImageDrawable(getResources().getDrawable(R.drawable.img_on));
		}else {
			img.setImageDrawable(getResources().getDrawable(R.drawable.img_off));
		}
    }

    @Override
    public void onDestroy() {
        super.onDestroy();
        // Stop the Bluetooth  services
        if (btService != null) btService.stop();
        Log.d(TAG, "onDestroy");
    }

    private void ensureDiscoverable() {
     
        if (mBluetoothAdapter.getScanMode() !=
            BluetoothAdapter.SCAN_MODE_CONNECTABLE_DISCOVERABLE) {
            Intent discoverableIntent = new Intent(BluetoothAdapter.ACTION_REQUEST_DISCOVERABLE);
            discoverableIntent.putExtra(BluetoothAdapter.EXTRA_DISCOVERABLE_DURATION, 300);
            startActivity(discoverableIntent);
        }
    }

   
    private void sendMessage(String message) {
       
        if (btService.getState() != BluetoothService.STATE_CONNECTED) {
            Toast.makeText(this, R.string.not_connected, Toast.LENGTH_SHORT).show();
            return;
        }

       
        if (message.length() > 0) {
          
            byte[] send = message.getBytes();
            btService.write(send);

           
            mOutStringBuffer.setLength(0);
       
        }
    }



    private final void setStatus(int resId) {
    	/*
        final ActionBar actionBar = getActionBar();
        actionBar.setSubtitle(resId);
        */
    }

    private final void setStatus(CharSequence subTitle) {
    	/*
        final ActionBar actionBar = getActionBar();
        actionBar.setSubtitle(subTitle);
        */
    }

    // The Handler that gets information back from the BluetoothService
    private final Handler mHandler = new Handler() {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
            case MESSAGE_STATE_CHANGE:
                Log.d(TAG, "MESSAGE_STATE_CHANGE: " + msg.arg1);
                switch (msg.arg1) {
                case BluetoothService.STATE_CONNECTED:
                    setStatus(getString(R.string.titleConnectedTo, mConnectedDeviceName));
                
                    break;
                case BluetoothService.STATE_CONNECTING:
                    setStatus(R.string.titleConnecting);
                    break;
                case BluetoothService.STATE_LISTEN:
                case BluetoothService.STATE_NONE:
                    setStatus(R.string.titleNotConnected);
                    break;
                }
                break;
            case MESSAGE_WRITE:
                byte[] writeBuf = (byte[]) msg.obj;
                // construct a string from the buffer
                String writeMessage = new String(writeBuf);
             
                break;
            case MESSAGE_READ:
                byte[] readBuf = (byte[]) msg.obj;
                // construct a string from the valid bytes in the buffer
                String readMessage = new String(readBuf, 0, msg.arg1);
               
                break;
            case MESSAGE_DEVICE_NAME:
                // save the connected device's name
                mConnectedDeviceName = msg.getData().getString(DEVICE_NAME);
                Toast.makeText(getApplicationContext(), "Connected to "
                               + mConnectedDeviceName, Toast.LENGTH_SHORT).show();
                break;
            case MESSAGE_TOAST:
                Toast.makeText(getApplicationContext(), msg.getData().getString(TOAST),
                               Toast.LENGTH_SHORT).show();
                break;
            }
        }
    };

    public void onActivityResult(int requestCode, int resultCode, Intent data) {
    	Log.d(TAG, "onActivityResult [" + resultCode + "]");
        switch (requestCode) {
        case REQUEST_CONNECT_DEVICE_SECURE:
            // When DeviceListActivity returns with a device to connect
            if (resultCode == Activity.RESULT_OK) {
                connectDevice(data);
            }
            break;

        case REQUEST_ENABLE_BT:
           
            if (resultCode == Activity.RESULT_OK) {
              
            	enableToSend();
            } else {
                Log.d(TAG, "BT not enabled");
                Toast.makeText(this, R.string.btNotEnabled, Toast.LENGTH_SHORT).show();
                finish();
            }
        }
    }

    private void connectDevice(Intent data) {
       
        String address = data.getExtras()
            .getString(DeviceListActivity.EXTRA_DEVICE_ADDRESS);
    
        BluetoothDevice device = mBluetoothAdapter.getRemoteDevice(address);
       
        btService.connect(device);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        MenuInflater inflater = getMenuInflater();
        inflater.inflate(R.menu.option_menu, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        Intent serverIntent = null;
        switch (item.getItemId()) {
        case R.id.secure_connect_scan:
          
            serverIntent = new Intent(this, DeviceListActivity.class);
            startActivityForResult(serverIntent, REQUEST_CONNECT_DEVICE_SECURE);
            return true;

        case R.id.discoverable:
           
            ensureDiscoverable();
            return true;
        }
        return false;
    }

}
